// author: Kwang-Hyun Park (akaii@kw.ac.kr)

#ifndef _LIBRARY_H_
#define _LIBRARY_H_

typedef void (*HAMSTER_DISPOSE_ALL)(void);
typedef void (*HAMSTER_WAIT)(int milliseconds);

typedef void (*HAMSTER_CREATE)(void);
typedef const char* (*HAMSTER_GET_NAME)(void);
typedef void (*HAMSTER_SET_NAME)(const char* name);
typedef const char* (*HAMSTER_GET_PORT_NAME)(void);
typedef const char* (*HAMSTER_GET_ADDRESS)(void);
typedef int (*HAMSTER_E)(int device_id);
typedef int (*HAMSTER_READ)(int device_id);
typedef int (*HAMSTER_READ_AT)(int device_id, int index);
typedef int (*HAMSTER_READ_ARRAY)(int device_id, int* data, int length);
typedef float (*HAMSTER_READ_FLOAT)(int device_id);
typedef float (*HAMSTER_READ_FLOAT_AT)(int device_id, int index);
typedef int (*HAMSTER_READ_FLOAT_ARRAY)(int device_id, float* data, int length);
typedef int (*HAMSTER_WRITE)(int device_id, int data);
typedef int (*HAMSTER_WRITE_AT)(int device_id, int index, int data);
typedef int (*HAMSTER_WRITE_ARRAY)(int device_id, const int* data, int length);
typedef int (*HAMSTER_WRITE_FLOAT)(int device_id, float data);
typedef int (*HAMSTER_WRITE_FLOAT_AT)(int device_id, int index, float data);
typedef int (*HAMSTER_WRITE_FLOAT_ARRAY)(int device_id, const float* data, int length);
typedef void (*HAMSTER_RESET)(void);
typedef void (*HAMSTER_SET_EXECUTABLE)(void (*execute)(int hamster_index));

typedef void (*HAMSTER_CREATE_MULTI)(int number_of_hamsters);
typedef const char* (*HAMSTER_GET_NAME_MULTI)(int hamster_index);
typedef void (*HAMSTER_SET_NAME_MULTI)(int hamster_index, const char* name);
typedef const char* (*HAMSTER_GET_PORT_NAME_MULTI)(int hamster_index);
typedef const char* (*HAMSTER_GET_ADDRESS_MULTI)(int hamster_index);
typedef int (*HAMSTER_E_MULTI)(int hamster_index, int device_id);
typedef int (*HAMSTER_READ_MULTI)(int hamster_index, int device_id);
typedef int (*HAMSTER_READ_AT_MULTI)(int hamster_index, int device_id, int index);
typedef int (*HAMSTER_READ_ARRAY_MULTI)(int hamster_index, int device_id, int* data, int length);
typedef float (*HAMSTER_READ_FLOAT_MULTI)(int hamster_index, int device_id);
typedef float (*HAMSTER_READ_FLOAT_AT_MULTI)(int hamster_index, int device_id, int index);
typedef int (*HAMSTER_READ_FLOAT_ARRAY_MULTI)(int hamster_index, int device_id, float* data, int length);
typedef int (*HAMSTER_WRITE_MULTI)(int hamster_index, int device_id, int data);
typedef int (*HAMSTER_WRITE_AT_MULTI)(int hamster_index, int device_id, int index, int data);
typedef int (*HAMSTER_WRITE_ARRAY_MULTI)(int hamster_index, int device_id, const int* data, int length);
typedef int (*HAMSTER_WRITE_FLOAT_MULTI)(int hamster_index, int device_id, float data);
typedef int (*HAMSTER_WRITE_FLOAT_AT_MULTI)(int hamster_index, int device_id, int index, float data);
typedef int (*HAMSTER_WRITE_FLOAT_ARRAY_MULTI)(int hamster_index, int device_id, const float* data, int length);
typedef void (*HAMSTER_RESET_MULTI)(int hamster_index);
typedef void (*HAMSTER_SET_EXECUTABLE_MULTI)(int hamster_index, void (*execute)(int hamster_index));

HAMSTER_DISPOSE_ALL hamster_dispose_all;
HAMSTER_WAIT hamster_wait;

HAMSTER_CREATE hamster_create;
HAMSTER_GET_NAME hamster_get_name;
HAMSTER_SET_NAME hamster_set_name;
HAMSTER_GET_PORT_NAME hamster_get_port_name;
HAMSTER_GET_ADDRESS hamster_get_address;
HAMSTER_E hamster_e;
HAMSTER_READ hamster_read;
HAMSTER_READ_AT hamster_read_at;
HAMSTER_READ_ARRAY hamster_read_array;
HAMSTER_READ_FLOAT hamster_read_float;
HAMSTER_READ_FLOAT_AT hamster_read_float_at;
HAMSTER_READ_FLOAT_ARRAY hamster_read_float_array;
HAMSTER_WRITE hamster_write;
HAMSTER_WRITE_AT hamster_write_at;
HAMSTER_WRITE_ARRAY hamster_write_array;
HAMSTER_WRITE_FLOAT hamster_write_float;
HAMSTER_WRITE_FLOAT_AT hamster_write_float_at;
HAMSTER_WRITE_FLOAT_ARRAY hamster_write_float_array;
HAMSTER_RESET hamster_reset;
HAMSTER_SET_EXECUTABLE hamster_set_executable;

HAMSTER_CREATE_MULTI hamster_create_multi;
HAMSTER_GET_NAME_MULTI hamster_get_name_multi;
HAMSTER_SET_NAME_MULTI hamster_set_name_multi;
HAMSTER_GET_PORT_NAME_MULTI hamster_get_port_name_multi;
HAMSTER_GET_ADDRESS_MULTI hamster_get_address_multi;
HAMSTER_E_MULTI hamster_e_multi;
HAMSTER_READ_MULTI hamster_read_multi;
HAMSTER_READ_AT_MULTI hamster_read_at_multi;
HAMSTER_READ_ARRAY_MULTI hamster_read_array_multi;
HAMSTER_READ_FLOAT_MULTI hamster_read_float_multi;
HAMSTER_READ_FLOAT_AT_MULTI hamster_read_float_at_multi;
HAMSTER_READ_FLOAT_ARRAY_MULTI hamster_read_float_array_multi;
HAMSTER_WRITE_MULTI hamster_write_multi;
HAMSTER_WRITE_AT_MULTI hamster_write_at_multi;
HAMSTER_WRITE_ARRAY_MULTI hamster_write_array_multi;
HAMSTER_WRITE_FLOAT_MULTI hamster_write_float_multi;
HAMSTER_WRITE_FLOAT_AT_MULTI hamster_write_float_at_multi;
HAMSTER_WRITE_FLOAT_ARRAY_MULTI hamster_write_float_array_multi;
HAMSTER_RESET_MULTI hamster_reset_multi;
HAMSTER_SET_EXECUTABLE_MULTI hamster_set_executable_multi;

void hamster_load_library(void);
void hamster_free_library(void);

#endif
